'use strict';

exports.__esModule = true;
exports.Nextion = undefined;

var _events = require('events');

var _system = require('./system');

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _fp = require('lodash/fp');

var _fp2 = _interopRequireDefault(_fp);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * @ignore
 */
var debug = (0, _debug2.default)('nextion:Nextion');

/**
 * Applies defaults to an object
 * @param {NextionOptions} obj - Defaults are applied to this object
 * @returns {NextionOptions} Options w/ defaults applied
 * @function
 * @private
 */
var applyDefaults = _fp2.default.defaults({
  // XXX: does nothing yet
  enhanced: true
});

/**
 * High-level abstraction for interacting with a Nextion device.
 * @extends {EventEmitter}
 */

var Nextion = exports.Nextion = function (_EventEmitter) {
  _inherits(Nextion, _EventEmitter);

  /**
   * Begins listening for data via a {@link UART} instance.
   * @param {UART} uart - {@link UART} instance
   * @param {Object|Function} [opts] - Options or `connectListener`
   * @param {Function} [connectListener] - Callback to run when listening for
   *   Nextion data
   * @emits {error} When binding via `uart` fails
   * @throws {ReferenceError} When `uart` is missing
   */
  function Nextion(uart) {
    var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var connectListener = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : _fp2.default.noop;

    _classCallCheck(this, Nextion);

    if (!uart) {
      throw new ReferenceError('Invalid parameters; Use Nextion.from(), Nextion.fromSerial(), or Nextion.fromPort()');
    }

    var _this = _possibleConstructorReturn(this, _EventEmitter.call(this));

    if (_fp2.default.isFunction(opts)) {
      connectListener = opts;
      opts = {};
    }

    _this.on('connected', function () {
      debug('Nextion ready!');
      connectListener();
    });

    /**
     * Options
     * @type {Object}
     * @private
     */
    _this.opts = applyDefaults(opts);

    /**
     * Internal UART instance
     * @type {UART}
     * @private
     */
    _this.uart = uart;

    // when a Nextion event occurs, re-emit it with event name
    uart.on('event', function (result) {
      debug(`Emitting event "${result.name}"`);
      _this.emit(result.name, result.data);
    }).on('disconnected', function () {
      debug('Nextion disconnected!');
    }).bind().then(function () {
      _this.emit('connected');
    }).catch(function (err) {
      _this.emit('error', err);
    });

    /**
     * System-level Nextion commands
     * @type {System}
     */
    _this.system = new _system.System(uart);

    debug('Instantiated');
    return _this;
  }

  /**
   * Sets a local or global variable on the current page to a value
   * @param {string} name - Name of variable
   * @param {*} [value] - New variable value
   * @returns {Promise<ResponseResult<*>,Error>} Result
   */


  Nextion.prototype.setValue = function setValue(name, value) {
    return this.uart.setValue(name, value);
  };

  /**
   * Sets a the value of a local component
   * @param {string} name - Name of component
   * @param {*} [value] - New component value
   * @returns {Promise<ResponseResult<*>,Error>} Result
   */


  Nextion.prototype.setComponentValue = function setComponentValue(name, value) {
    return this.setVariableValue(`${name}.val`, value);
  };

  /**
   * Get a value
   * @param {string} name - Name; can be `varName.val` or `component.txt`, etc.
   * @returns {Promise<ResponseResult<StringData|NumericData>,Error>} String or
   *   numeric data response (depending on variable's type)
   */


  Nextion.prototype.getValue = function getValue(name) {
    return this.uart.getValue(name);
  };

  /**
   * Closes connection to Nextion device.
   * @returns {Promise<Nextion>} This instance
   */


  Nextion.prototype.close = function close() {
    var _this2 = this;

    return this.uart.unbind().then(function () {
      return _this2;
    });
  };

  return Nextion;
}(_events.EventEmitter);

Nextion.prototype.setVariableValue = Nextion.prototype.setValue;
//# sourceMappingURL=nextion.js.map