'use strict';

exports.__esModule = true;
exports.System = exports.SYSTEM_VARIABLES = undefined;

var _fp = require('lodash/fp');

var _fp2 = _interopRequireDefault(_fp);

var _events = require('events');

var _util = require('./util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * These are the names of the internal system variables.
 * @type {string[]}
 * @private
 */
var SYSTEM_VARIABLES = exports.SYSTEM_VARIABLES = ['sys0', 'sys1', 'sys2'];

/**
 * System-level Nextion device functionality.
 * @extends {EventEmitter}
 */

var System = exports.System = function (_EventEmitter) {
  _inherits(System, _EventEmitter);

  /**
   * Creates a {@link System} instance.
   * @param {UART} uart - UART instance.
   */
  function System(uart) {
    _classCallCheck(this, System);

    var _this = _possibleConstructorReturn(this, _EventEmitter.call(this));

    _this.uart = uart;
    return _this;
  }

  /**
   * Sets value of system variable.
   * @param {string} name - Name of system variable; either `sys0`, `sys1`, or
   *   `sys2`.
   * @param {number} value - Must be an unsigned integer less than or equal to
   *   `4294967295`.
   * @returns {Promise<ResponseResult,TypeError>} Response or {@link TypeError}
   *   if invalid `name` or `value`
   */


  System.prototype.setSystemVariable = function setSystemVariable(name, value) {
    var _this2 = this;

    return Promise.resolve().then(function () {
      if (!_fp2.default.includes(name, SYSTEM_VARIABLES)) {
        throw new TypeError(`"name" must be one of: ${SYSTEM_VARIABLES.join(', ')}, but found "${name}"`);
      }
      if (!(0, _util.isUnsignedInteger)(value)) {
        throw new TypeError(`"value" must be an unsigned integer <= ${_util.MAX_INT}; got ${value}`);
      }
      return _this2.uart.setValue(name, value);
    });
  };

  /**
   * Sets the range of results returned by {@link System#random}.
   * @param {number} [min=0] - Unsigned integer; no greater than `max`
   * @param {number} [max=4294967295] - Unsigned integer; no less than `min`
   * @returns {Promise.<ResponseResult,TypeError>} Response or {@link
    *   TypeError} if invalid `min` or `max` value
   */


  System.prototype.setRandomRange = function setRandomRange() {
    var _this3 = this;

    var min = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
    var max = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;

    return Promise.resolve().then(function () {
      if (!(0, _util.isUnsignedInteger)(min)) {
        throw new TypeError(`"min" must be an unsigned integer <= ${_util.MAX_INT}`);
      }
      if (!(0, _util.isUnsignedInteger)(max)) {
        throw new TypeError(`"max" must be an unsigned integer <= ${_util.MAX_INT}`);
      }
      if (min > max) {
        throw new TypeError('"min" cannot be greater than "max"');
      }

      return _this3.uart.request(`ranset ${min},${max}`);
    });
  };

  /**
   * Gets a random value, optionally setting the allowed range via {@link
    * System#setRandomRange}.
   * @param {number} [min=0] - Unsigned integer; no greater than `max`
   * @param {number} [max=0] - Unsigned integer; no less than `min`
   * @returns {Promise<ResponseResult,TypeError>} Result or error if invalid
   *   range supplied
   */


  System.prototype.random = function random() {
    var _this4 = this;

    var min = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
    var max = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

    return Promise.resolve(function () {
      if (min || max) {
        return _this4.setRandomRange(min, max);
      }
    }).then(function () {
      return _this4.uart.request('rand');
    });
  };

  /**
   * Set sleep timer when not touched.
   * @param {number} ms - Sleep after `ms` milliseconds of no touching. Rounded
   *   to closest second.
   * @returns {Promise.<ResponseResult, Error>} Result
   */


  System.prototype.setNoTouchSleepTimer = function setNoTouchSleepTimer() {
    var ms = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;

    return this.uart.setValue('thsp', Math.floor(ms / 1000));
  };

  /**
   * Puts Nextion to sleep.
   * @see {@link System#wake}
   * @param {boolean} [shouldSleep=true] Sleep if `true`; wake if `false`.
   * @returns {Promise.<ResponseResult, Error>} Result
   */


  System.prototype.sleep = function sleep() {
    var shouldSleep = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

    return this.uart.setValue('sleep', shouldSleep);
  };

  /**
   * Wakes Nextion from sleep.
   * @see {@link System#sleep}
   * @returns {Promise.<ResponseResult, Error>} Result
   */


  System.prototype.wake = function wake() {
    return this.sleep(false);
  };

  return System;
}(_events.EventEmitter);
//# sourceMappingURL=system.js.map